@@ -23,7 +23,7 @@ use Data::Dumper;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '0.47';
+$VERSION = '0.49';
 
 =head1 NAME
 
@@ -606,7 +606,7 @@ sub getcmd {
       }
       for my $opt (sort keys %{$merged->{list}}) {
          next if $opt eq 'literal';
-         for my $val (@{$merged->{$opt}}) {
+         for my $val (@{$merged->{list}{$opt}}) {
             push @cmd, "--$opt=$val";
          }
       }
@@ -686,7 +686,22 @@ sub exec {
    my ($cmd, $infun, $outfun, $errfun, $debug) = $self->getcmd(@_);
    return unless $cmd;
    warn "exec: @$cmd\n" if $debug;
-   run3($cmd, $infun, \my $stdout, \my $stderr);
+   my $input;
+   if (ref $infun eq 'CODE') {
+      my $pid = open my $fh, '-|';
+      if ($pid) {    # parent grabs output
+         my @in = <$fh>;
+         close $fh;
+         chomp @in;
+         $input = \@in;
+      } else {    # child runs infun
+         &{$infun};
+         exit;
+      }
+   } else {
+      $input = $infun;
+   }
+   run3($cmd, $input, \my $stdout, \my $stderr);
    $self->{_lastcmd}    = $cmd;
    $self->{_realstatus} = $?;
    $self->{_status}     = $? & 127 ? $? & 127 : $? >> 8;
